@@ -48,6 +48,7 @@
 #include "cache_pool.h"
 
 struct conn_pool;
+static inline int vcp_cmp(const struct conn_pool *a, const struct conn_pool *b);
 
 /*--------------------------------------------------------------------
  */
@@ -88,7 +89,7 @@ struct conn_pool {
 	struct vrt_endpoint			*endpoint;
 	char					ident[VSHA256_DIGEST_LENGTH];
 
-	VTAILQ_ENTRY(conn_pool)			list;
+	VRBT_ENTRY(conn_pool)			entry;
 	int					refcnt;
 	struct lock				mtx;
 
@@ -104,10 +105,20 @@ struct conn_pool {
 };
 
 static struct lock conn_pools_mtx;
+static struct lock dead_pools_mtx;
 
-static VTAILQ_HEAD(, conn_pool)	conn_pools =
-    VTAILQ_HEAD_INITIALIZER(conn_pools);
+VRBT_HEAD(vrb, conn_pool);
+VRBT_GENERATE_REMOVE_COLOR(vrb, conn_pool, entry, static)
+VRBT_GENERATE_REMOVE(vrb, conn_pool, entry, static)
+VRBT_GENERATE_FIND(vrb, conn_pool, entry, vcp_cmp, static)
+VRBT_GENERATE_INSERT_COLOR(vrb, conn_pool, entry, static)
+VRBT_GENERATE_INSERT_FINISH(vrb, conn_pool, entry, static)
+VRBT_GENERATE_INSERT(vrb, conn_pool, entry, vcp_cmp, static)
+VRBT_GENERATE_NEXT(vrb, conn_pool, entry, static)
+VRBT_GENERATE_MINMAX(vrb, conn_pool, entry, static)
 
+static struct vrb conn_pools = VRBT_INITIALIZER(&conn_pools);
+static struct vrb dead_pools = VRBT_INITIALIZER(&dying_cps);
 
 /*--------------------------------------------------------------------
  */
@@ -144,6 +155,15 @@ PFD_RemoteName(const struct pfd *p, char *abuf, unsigned alen, char *pbuf,
 	p->conn_pool->methods->remote_name(p, abuf, alen, pbuf, plen);
 }
 
+/*--------------------------------------------------------------------
+ */
+
+static inline int
+vcp_cmp(const struct conn_pool *a, const struct conn_pool *b)
+{
+	return (memcmp(a->ident, b->ident, sizeof b->ident));
+}
+
 /*--------------------------------------------------------------------
  * Waiter-handler
  */
@@ -203,6 +223,22 @@ VCP_AddRef(struct conn_pool *cp)
 	Lck_Unlock(&conn_pools_mtx);
 }
 
+/*--------------------------------------------------------------------
+ */
+
+static void
+vcp_destroy(struct conn_pool **cpp)
+{
+	struct conn_pool *cp;
+
+	TAKE_OBJ_NOTNULL(cp, cpp, CONN_POOL_MAGIC);
+	AZ(cp->n_conn);
+	AZ(cp->n_kill);
+	Lck_Delete(&cp->mtx);
+	free(cp->endpoint);
+	FREE_OBJ(cp);
+}
+
 /*--------------------------------------------------------------------
  * Release Conn pool, destroy if last reference.
  */
@@ -212,6 +248,7 @@ VCP_Rel(struct conn_pool **cpp)
 {
 	struct conn_pool *cp;
 	struct pfd *pfd, *pfd2;
+	int n_kill;
 
 	TAKE_OBJ_NOTNULL(cp, cpp, CONN_POOL_MAGIC);
 
@@ -222,7 +259,7 @@ VCP_Rel(struct conn_pool **cpp)
 		return;
 	}
 	AZ(cp->n_used);
-	VTAILQ_REMOVE(&conn_pools, cp, list);
+	VRBT_REMOVE(vrb, &conn_pools, cp);
 	Lck_Unlock(&conn_pools_mtx);
 
 	Lck_Lock(&cp->mtx);
@@ -234,17 +271,62 @@ VCP_Rel(struct conn_pool **cpp)
 		(void)shutdown(pfd->fd, SHUT_RDWR);
 		cp->n_kill++;
 	}
-	while (cp->n_kill) {
-		Lck_Unlock(&cp->mtx);
-		(void)usleep(20000);
-		Lck_Lock(&cp->mtx);
-	}
+	n_kill = cp->n_kill;
 	Lck_Unlock(&cp->mtx);
-	Lck_Delete(&cp->mtx);
-	AZ(cp->n_conn);
-	AZ(cp->n_kill);
-	free(cp->endpoint);
-	FREE_OBJ(cp);
+	if (n_kill == 0) {
+		vcp_destroy(&cp);
+		return;
+	}
+	Lck_Lock(&dead_pools_mtx);
+	/*
+	 * Here we reuse cp's entry but it will probably not be correctly
+	 * indexed because of the hack in VCP_RelPoll
+	 */
+	VRBT_INSERT(vrb, &dead_pools, cp);
+	Lck_Unlock(&dead_pools_mtx);
+}
+
+void
+VCP_RelPoll(void)
+{
+	struct vrb dead;
+	struct conn_pool *cp, *cp2;
+	int n_kill;
+
+	ASSERT_CLI();
+
+	Lck_Lock(&dead_pools_mtx);
+	if (VRBT_EMPTY(&dead_pools)) {
+		Lck_Unlock(&dead_pools_mtx);
+		return;
+	}
+	dead = dead_pools;
+	VRBT_INIT(&dead_pools);
+	Lck_Unlock(&dead_pools_mtx);
+
+	VRBT_FOREACH_SAFE(cp, vrb, &dead, cp2) {
+		CHECK_OBJ_NOTNULL(cp, CONN_POOL_MAGIC);
+		Lck_Lock(&cp->mtx);
+		n_kill = cp->n_kill;
+		Lck_Unlock(&cp->mtx);
+		if (n_kill > 0)
+			continue;
+		VRBT_REMOVE(vrb, &dead, cp);
+		vcp_destroy(&cp);
+	}
+
+	if (VRBT_EMPTY(&dead))
+		return;
+
+	Lck_Lock(&dead_pools_mtx);
+	/*
+	 * The following insertion will most likely result in an
+	 * unordered tree, but in this case it does not matter
+	 * as we just want to iterate over all the elements
+	 * in the tree in order to delete them.
+	 */
+	VRBT_INSERT(vrb, &dead_pools, dead.rbh_root);
+	Lck_Unlock(&dead_pools_mtx);
 }
 
 /*--------------------------------------------------------------------
@@ -304,7 +386,7 @@ VCP_Recycle(const struct worker *wrk, struct pfd **pfdp)
 		 * have N+1 backend connections rather than N, which is
 		 * entirely harmless.
 		 */
-		(void)usleep(10000);
+		VTIM_sleep(0.01);
 	}
 }
 
@@ -568,6 +650,7 @@ void
 VCP_Init(void)
 {
 	Lck_New(&conn_pools_mtx, lck_conn_pool);
+	Lck_New(&dead_pools_mtx, lck_dead_pool);
 }
 
 /**********************************************************************/
@@ -714,13 +797,6 @@ VCP_Ref(const struct vrt_endpoint *vep, const char *ident)
 	}
 	VSHA256_Final(digest, cx);
 
-	/*
-	 * In heavy use of dynamic backends, traversing this list
-	 * can become expensive.  In order to not do so twice we
-	 * pessimistically create the necessary pool, and discard
-	 * it on a hit. (XXX: Consider hash or tree ?)
-	 */
-
 	ALLOC_OBJ(cp, CONN_POOL_MAGIC);
 	AN(cp);
 	cp->refcnt = 1;
@@ -737,16 +813,14 @@ VCP_Ref(const struct vrt_endpoint *vep, const char *ident)
 
 	CHECK_OBJ_NOTNULL(cp, CONN_POOL_MAGIC);
 	Lck_Lock(&conn_pools_mtx);
-	VTAILQ_FOREACH(cp2, &conn_pools, list) {
-		CHECK_OBJ_NOTNULL(cp2, CONN_POOL_MAGIC);
-		assert(cp2->refcnt > 0);
-		if (!memcmp(digest, cp2->ident, sizeof cp2->ident))
-			break;
-	}
+	cp2 = VRBT_FIND(vrb, &conn_pools, cp);
 	if (cp2 == NULL)
-		VTAILQ_INSERT_HEAD(&conn_pools, cp, list);
-	else
+		AZ(VRBT_INSERT(vrb, &conn_pools, cp));
+	else {
+		CHECK_OBJ(cp2, CONN_POOL_MAGIC);
+		assert(cp2->refcnt > 0);
 		cp2->refcnt++;
+	}
 	Lck_Unlock(&conn_pools_mtx);
 
 	if (cp2 == NULL) {
