@@ -42,6 +42,7 @@
 
 #include "vtc.h"
 
+#include "vsa.h"
 #include "vss.h"
 #include "vtcp.h"
 #include "vus.h"
@@ -52,19 +53,16 @@ struct client {
 	char			*name;
 	struct vtclog		*vl;
 	VTAILQ_ENTRY(client)	list;
+	struct vtc_sess		*vsp;
 
 	char			*spec;
 
 	char			connect[256];
-	const char		*addr;
-	int			rcvbuf;
+	char			*addr;
 
 	char			*proxy_spec;
 	int			proxy_version;
 
-	int			repeat;
-	unsigned		keepalive;
-
 	unsigned		running;
 	pthread_t		tp;
 };
@@ -76,28 +74,39 @@ static VTAILQ_HEAD(, client)	clients = VTAILQ_HEAD_INITIALIZER(clients);
  */
 
 static void
-client_proxy(struct vtclog *vl, int fd, int version, const char *spec)
+client_proxy(struct vtclog *vl, int fd, int version, const char *speca)
 {
-	struct suckaddr *sac, *sas;
-	char *p, *p2;
+	const struct suckaddr *sac, *sas;
+	char *spec, *save, *p;
+	struct vsb *tlv;
 
-	p = strdup(spec);
+	spec = strdup(speca);
+	AN(spec);
+	save = NULL;
+
+	p = strtok_r(spec, " ", &save);
 	AN(p);
-	p2 = strchr(p, ' ');
-	AN(p2);
-	*p2++ = '\0';
-
 	sac = VSS_ResolveOne(NULL, p, NULL, 0, SOCK_STREAM, AI_PASSIVE);
 	if (sac == NULL)
 		vtc_fatal(vl, "Could not resolve client address");
-	sas = VSS_ResolveOne(NULL, p2, NULL, 0, SOCK_STREAM, AI_PASSIVE);
+
+	p = strtok_r(NULL, " ", &save);
+	AN(p);
+	sas = VSS_ResolveOne(NULL, p, NULL, 0, SOCK_STREAM, AI_PASSIVE);
 	if (sas == NULL)
-		vtc_fatal(vl, "Could not resolve server address");
-	if (vtc_send_proxy(fd, version, sac, sas))
+		vtc_fatal(vl, "Could not resolve client address");
+
+	tlv = VSB_new_auto();
+	while ((p = strtok_r(NULL, " ", &save)) != NULL)
+		vtc_proxy_tlv(vl, tlv, p);
+
+	free(spec);
+
+	AZ(VSB_finish(tlv));
+	if (vtc_send_proxy(fd, version, sac, sas, tlv))
 		vtc_fatal(vl, "Write failed: %s", strerror(errno));
-	free(p);
-	free(sac);
-	free(sas);
+	VSA_free(&sac);
+	VSA_free(&sas);
 }
 
 /**********************************************************************
@@ -109,7 +118,7 @@ client_tcp_connect(struct vtclog *vl, const char *addr, double tmo,
 		   const char **errp)
 {
 	int fd;
-	char mabuf[32], mpbuf[32];
+	char mabuf[VTCP_ADDRBUFSIZE], mpbuf[VTCP_PORTBUFSIZE];
 
 	AN(addr);
 	AN(errp);
@@ -129,7 +138,9 @@ uds_open(void *priv, const struct sockaddr_un *uds)
 	double *p;
 	int s, i, tmo;
 	struct pollfd fds[1];
-	socklen_t sl = sizeof(*uds);
+	socklen_t sl;
+
+	sl = VUS_socklen(uds);
 
 	AN(priv);
 	AN(uds);
@@ -144,7 +155,7 @@ uds_open(void *priv, const struct sockaddr_un *uds)
 	VTCP_nonblocking(s);
 	i = connect(s, (const void*)uds, sl);
 	if (i == 0)
-		return(s);
+		return (s);
 	if (errno != EINPROGRESS) {
 		closefd(&s);
 		return (-1);
@@ -189,7 +200,7 @@ client_connect(struct vtclog *vl, struct client *c)
 	int fd;
 
 	vtc_log(vl, 3, "Connect to %s", c->addr);
-	if (*c->addr == '/')
+	if (VUS_is(c->addr))
 		fd = client_uds_connect(vl, c->addr, 10., &err);
 	else
 		fd = client_tcp_connect(vl, c->addr, 10., &err);
@@ -207,54 +218,21 @@ client_connect(struct vtclog *vl, struct client *c)
  * Client thread
  */
 
-static void *
-client_thread(void *priv)
+static int
+client_conn(void *priv, struct vtclog *vl)
 {
 	struct client *c;
-	struct vtclog *vl;
-	int fd;
-	int i;
-	struct vsb *vsb;
 
 	CAST_OBJ_NOTNULL(c, priv, CLIENT_MAGIC);
-	AN(*c->connect);
+	return (client_connect(vl, c));
+}
 
-	vl = vtc_logopen(c->name);
-	pthread_cleanup_push(vtc_logclose, vl);
-
-	vsb = macro_expand(vl, c->connect);
-	AN(vsb);
-#if !defined(__sun)
-	pthread_cleanup_push((void (*)(void *))VSB_delete, vsb);
-#endif
-	c->addr = VSB_data(vsb);
-
-	if (c->repeat == 0)
-		c->repeat = 1;
-	if (c->repeat != 1)
-		vtc_log(vl, 2, "Started (%u iterations%s)", c->repeat,
-			c->keepalive ? " using keepalive" : "");
-	for (i = 0; i < c->repeat; i++) {
-		fd = client_connect(vl, c);
-
-		if (! c->keepalive)
-			fd = http_process(vl, c->spec, fd, NULL, c->addr,
-			    c->rcvbuf);
-		else
-			while (fd >= 0 && i++ < c->repeat)
-				fd = http_process(vl, c->spec, fd, NULL,
-				    c->addr, c->rcvbuf);
-		vtc_log(vl, 3, "closing fd %d", fd);
-		VTCP_close(&fd);
-	}
-	vtc_log(vl, 2, "Ending");
-#if !defined(__sun)
-	pthread_cleanup_pop(0);
-#endif
-	pthread_cleanup_pop(0);
-	VSB_delete(vsb);
-	vtc_logclose(vl);
-	return (NULL);
+static void
+client_disc(void *priv, struct vtclog *vl, int *fdp)
+{
+	(void)priv;
+	vtc_log(vl, 3, "closing fd %d", *fdp);
+	VTCP_close(fdp);
 }
 
 /**********************************************************************
@@ -269,8 +247,10 @@ client_new(const char *name)
 	ALLOC_OBJ(c, CLIENT_MAGIC);
 	AN(c);
 	REPLACE(c->name, name);
-	c->vl = vtc_logopen(name);
+	c->vl = vtc_logopen("%s", name);
 	AN(c->vl);
+	c->vsp = Sess_New(c->vl, name);
+	AN(c->vsp);
 
 	bprintf(c->connect, "%s", "${v1_sock}");
 	VTAILQ_INSERT_TAIL(&clients, c, list);
@@ -286,9 +266,11 @@ client_delete(struct client *c)
 {
 
 	CHECK_OBJ_NOTNULL(c, CLIENT_MAGIC);
+	Sess_Destroy(&c->vsp);
 	vtc_logclose(c->vl);
 	free(c->spec);
 	free(c->name);
+	free(c->addr);
 	free(c->proxy_spec);
 	/* XXX: MEMLEAK (?)*/
 	FREE_OBJ(c);
@@ -301,11 +283,24 @@ client_delete(struct client *c)
 static void
 client_start(struct client *c)
 {
+	struct vsb *vsb;
 
 	CHECK_OBJ_NOTNULL(c, CLIENT_MAGIC);
 	vtc_log(c->vl, 2, "Starting client");
-	AZ(pthread_create(&c->tp, NULL, client_thread, c));
 	c->running = 1;
+	vsb = macro_expand(c->vl, c->connect);
+	AN(vsb);
+	REPLACE(c->addr, VSB_data(vsb));
+	VSB_destroy(&vsb);
+	c->tp = Sess_Start_Thread(
+	    c,
+	    c->vsp,
+	    client_conn,
+	    client_disc,
+	    c->addr,
+	    NULL,
+	    c->spec
+	);
 }
 
 /**********************************************************************
@@ -319,7 +314,7 @@ client_wait(struct client *c)
 
 	CHECK_OBJ_NOTNULL(c, CLIENT_MAGIC);
 	vtc_log(c->vl, 2, "Waiting for client");
-	AZ(pthread_join(c->tp, &res));
+	PTOK(pthread_join(c->tp, &res));
 	if (res != NULL)
 		vtc_fatal(c->vl, "Client returned \"%s\"", (char *)res);
 	c->tp = 0;
@@ -349,7 +344,6 @@ cmd_client(CMD_ARGS)
 	struct client *c, *c2;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL) {
 		/* Reset and free */
@@ -386,6 +380,10 @@ cmd_client(CMD_ARGS)
 		if (c->running)
 			client_wait(c);
 
+		AZ(c->running);
+		if (Sess_GetOpt(c->vsp, &av))
+			continue;
+
 		if (!strcmp(*av, "-connect")) {
 			bprintf(c->connect, "%s", av[1]);
 			av++;
@@ -403,20 +401,6 @@ cmd_client(CMD_ARGS)
 			av++;
 			continue;
 		}
-		if (!strcmp(*av, "-repeat")) {
-			c->repeat = atoi(av[1]);
-			av++;
-			continue;
-		}
-		if (!strcmp(*av, "-keepalive")) {
-			c->keepalive = 1;
-			continue;
-		}
-		if (!strcmp(*av, "-rcvbuf")) {
-			c->rcvbuf = atoi(av[1]);
-			av++;
-			continue;
-		}
 		if (!strcmp(*av, "-start")) {
 			client_start(c);
 			continue;
