@@ -252,8 +252,6 @@ struct worker {
 
 	vtim_real		lastused;
 
-	struct v1l		*v1l;
-
 	pthread_cond_t		cond;
 
 	struct ws		aws[1];
@@ -326,6 +324,8 @@ enum oc_flags {
 #include "tbl/oc_flags.h"
 };
 
+#define OC_F_TRANSIENT (OC_F_PRIVATE | OC_F_HFM | OC_F_HFP)
+
 enum oc_exp_flags {
 #define OC_EXP_FLAG(U, l, v)	OC_EF_##U = v,
 #include "tbl/oc_exp_flags.h"
@@ -383,17 +383,17 @@ struct busyobj {
 
 	char			*end;
 
-	/*
-	 * All fields from retries and down are zeroed when the busyobj
-	 * is recycled.
-	 */
+	unsigned		max_retries;
 	unsigned		retries;
 	struct req		*req;
 	struct sess		*sp;
 	struct worker		*wrk;
 
+	/* beresp.body */
 	struct vfp_ctx		*vfc;
 	const char		*vfp_filter_list;
+	/* bereq.body */
+	const char		*vdp_filter_list;
 
 	struct ws		ws[1];
 	uintptr_t		ws_bo;
@@ -410,15 +410,21 @@ struct busyobj {
 
 	struct pool_task	fetch_task[1];
 
+	const char		*err_reason;
+	enum director_state_e	director_state;
+	uint16_t		err_code;
+
 #define BERESP_FLAG(l, r, w, f, d) unsigned	l:1;
 #define BEREQ_FLAG(l, r, w, d) BERESP_FLAG(l, r, w, 0, d)
 #include "tbl/bereq_flags.h"
 #include "tbl/beresp_flags.h"
 
+
 	/* Timeouts */
 	vtim_dur		connect_timeout;
 	vtim_dur		first_byte_timeout;
 	vtim_dur		between_bytes_timeout;
+	vtim_dur		task_deadline;
 
 	/* Timers */
 	vtim_real		t_first;	/* First timestamp logged */
@@ -431,7 +437,6 @@ struct busyobj {
 	const struct stevedore	*storage;
 	const struct director	*director_req;
 	const struct director	*director_resp;
-	enum director_state_e	director_state;
 	struct vcl		*vcl;
 
 	struct vsl_log		vsl[1];
@@ -439,12 +444,12 @@ struct busyobj {
 	uint8_t			digest[DIGEST_LEN];
 	struct vrt_privs	privs[1];
 
-	uint16_t		err_code;
-	const char		*err_reason;
-
 	const char		*client_identity;
 };
 
+#define BUSYOBJ_TMO(bo, pfx, tmo)					\
+	(isnan((bo)->tmo) ? cache_param->pfx##tmo : (bo)->tmo)
+
 
 /*--------------------------------------------------------------------*/
 
@@ -460,23 +465,19 @@ struct req {
 	unsigned		magic;
 #define REQ_MAGIC		0xfb4abf6d
 
+	unsigned		esi_level;
 	body_status_t		req_body_status;
 	stream_close_t		doclose;
 	unsigned		restarts;
-	unsigned		esi_level;
-
-	/* Delivery mode */
-	unsigned		res_mode;
-#define RES_ESI			(1<<4)
-#define RES_PIPE		(1<<7)
+	unsigned		max_restarts;
 
 	const struct req_step	*req_step;
 	struct reqtop		*top;	/* esi_level == 0 request */
 
+	uint16_t		err_code;
 #define REQ_FLAG(l, r, w, d) unsigned	l:1;
 #include "tbl/req_flags.h"
 
-	uint16_t		err_code;
 	const char		*err_reason;
 
 	struct sess		*sp;
@@ -530,10 +531,13 @@ struct req {
 	struct ws		ws[1];
 	struct objcore		*objcore;
 	struct objcore		*stale_oc;
+	struct boc		*boc;		/* valid during cnt_transmit */
 
-	/* Deliver pipeline */
+	/* resp.body */
 	struct vdp_ctx		*vdc;
-	const char		*filter_list;
+	const char		*vdp_filter_list;
+	/* req.body */
+	const char		*vfp_filter_list;
 
 	/* Transaction VSL buffer */
 	struct vsl_log		vsl[1];
