@@ -649,6 +649,23 @@ PARAM_SIMPLE(
 	"the initial SETTINGS frame."
 )
 
+PARAM_SIMPLE(
+	/* name */	http_req_overflow_status,
+	/* type */	uint_orzero,
+	/* min */	"400",
+	/* max */	"499",
+	/* def */	"0",
+	/* units */	"HTTP status code or 0 to disable",
+	/* descr */
+	"HTTP status code to be returned if http_req_size is exceeded. "
+	"The default value of 0 closes the connection silently without "
+	"sending a HTTP response.\n"
+	"Note that there is no standard HTTP status which exactly matches "
+	"the implementation of http_req_size. 414 applies to the URL only, "
+	"while 413 applies to the request body. 400 is probably the least "
+	"incorrect alternative value to sending no response at all (0)."
+)
+
 PARAM_SIMPLE(
 	/* name */	http_resp_hdr_len,
 	/* type */	bytes_u,
@@ -721,6 +738,20 @@ PARAM_SIMPLE(
 	/* flags */	EXPERIMENTAL
 )
 
+PARAM_SIMPLE(
+	/* name */	ban_any_variant,
+	/* type */	uint,
+	/* min */	"0",
+	/* max */	NULL,
+	/* def */	"10000",
+	/* units */	"checks",
+	/* descr */
+	"Maximum number of possibly non matching variants that we evaluate "
+	"against the ban list during a lookup.\n"
+	"Setting this to 0 means that only the matching variants will be "
+	"evaluated against the current ban list."
+)
+
 PARAM_SIMPLE(
 	/* name */	max_esi_depth,
 	/* type */	uint,
@@ -740,7 +771,8 @@ PARAM_SIMPLE(
 	/* def */	"4",
 	/* units */	"restarts",
 	/* descr */
-	"Upper limit on how many times a request can restart."
+	"Upper limit on how many times a request can restart.",
+	/* flags */	DELAYED_EFFECT
 )
 
 PARAM_SIMPLE(
@@ -751,7 +783,8 @@ PARAM_SIMPLE(
 	/* def */	"4",
 	/* units */	"retries",
 	/* descr */
-	"Upper limit on how many times a backend fetch can retry."
+	"Upper limit on how many times a backend fetch can retry.",
+	/* flags */	DELAYED_EFFECT
 )
 
 PARAM_SIMPLE(
@@ -950,7 +983,7 @@ PARAM_SIMPLE(
 	/* name */	tcp_keepalive_intvl,
 	/* type */	duration,
 	/* min */	"1",
-	/* max */	"100",
+	/* max */	"150",
 	/* def */	NULL,
 	/* units */	"seconds",
 	/* descr */
@@ -983,7 +1016,7 @@ PARAM_SIMPLE(
 	/* name */	tcp_keepalive_time,
 	/* type */	duration,
 	/* min */	"1",
-	/* max */	"7200",
+	/* max */	"14400",
 	/* def */	NULL,
 	/* units */	"seconds",
 	/* descr */
@@ -1800,7 +1833,8 @@ PARAM_STRING(
 	"VCL files in both the system configuration and shared "
 	"data directories to allow packages to drop their VCL "
 	"files in a standard location where relative includes "
-	"would work.",
+	"would work. Includes using +glob cannot be searched "
+	"in vcl_path.",
 	/* flags */	BUILD_OPTIONS,
 	/* dyn_min_reason */	NULL,
 	/* dyn_max_reason */	NULL,
