@@ -46,9 +46,9 @@
 #  error "include vdef.h before vrt.h"
 #endif
 
-#define VRT_MAJOR_VERSION	20U
+#define VRT_MAJOR_VERSION	21U
 
-#define VRT_MINOR_VERSION	1U
+#define VRT_MINOR_VERSION	0U
 
 /***********************************************************************
  * Major and minor VRT API versions.
@@ -57,7 +57,14 @@
  * Whenever something is deleted or changed in a way which is not
  * binary/load-time compatible, increment MAJOR version
  *
- * 20.1 (2024-11-08)
+ * 21.0 (2025-03-17)
+ *	VRT_u_req_grace() added
+ *	VRT_u_req_ttl() added
+ *	VRT_r_req_filters() added
+ *	VRT_l_req_filters() added
+ *	VRT_r_bereq_filters() added
+ *	VRT_l_bereq_filters() added
+ * 20.1 (2024-11-08 7.6.1)
  *	VDI_EVENT_SICK added to enum vcl_event_e
  * 20.0 (2024-09-13)
  *	struct vrt_backend.backend_wait_timeout added
@@ -492,6 +499,8 @@ struct vmod_data {
 	const char			*proto;
 	const char			*json;
 	const char			*abi;
+	const char			*vcs;
+	const char			*version;
 };
 
 /***********************************************************************
