@@ -243,8 +243,20 @@ int __llvm_gcov_flush(void);
 
 #if __GNUC_PREREQ__(4, 3) || defined(__clang__)
 #  define v_cold_    __attribute__((cold))
+#  define v_used_(x) __attribute__((x, used))
 #else
 #  define v_cold_
+#  define v_used_(x)
+#endif
+
+#if defined __has_attribute
+#  if __has_attribute(counted_by)
+#    define v_counted_by_(field) __attribute__((counted_by(field)))
+#  endif
+#endif
+
+#ifndef v_counted_by_
+#  define v_counted_by_(field)
 #endif
 
 /* VTIM API overhaul WIP */
@@ -263,6 +275,9 @@ typedef struct {
 
 #define Tcheck(t)	do { (void)pdiff((t).b, (t).e); } while (0)
 #define Tlen(t)		(pdiff((t).b, (t).e))
+#define Tstr(s)		(/*lint -e(446)*/ (txt){(s), (s) + strlen(s)})
+#define Tstrcmp(t, s)	(strncmp((t).b, (s), Tlen(t)))
+#define Tforeach(c, t)	for ((c) = (t).b; (c) < (t).e; (c)++)
 
 /* #3020 dummy definitions until PR is merged*/
 #define LIKELY(x)	(x)
@@ -315,7 +330,7 @@ typedef struct {
 #  error "include vdef.h before vrt.h"
 #endif
 
-#define VRT_MAJOR_VERSION	19U
+#define VRT_MAJOR_VERSION	21U
 
 #define VRT_MINOR_VERSION	0U
 
@@ -326,6 +341,20 @@ typedef struct {
  * Whenever something is deleted or changed in a way which is not
  * binary/load-time compatible, increment MAJOR version
  *
+ * 21.0 (2025-03-17)
+ *	VRT_u_req_grace() added
+ *	VRT_u_req_ttl() added
+ *	VRT_r_req_filters() added
+ *	VRT_l_req_filters() added
+ *	VRT_r_bereq_filters() added
+ *	VRT_l_bereq_filters() added
+ * 20.1 (2024-11-08 7.6.1)
+ *	VDI_EVENT_SICK added to enum vcl_event_e
+ * 20.0 (2024-09-13)
+ *	struct vrt_backend.backend_wait_timeout added
+ *	struct vrt_backend.backend_wait_limit  added
+ * 19.1 (2024-05-27)
+ *	[cache_varnishd.h] ObjWaitExtend() gained statep argument
  * 19.0 (2024-03-18)
  *	[cache.h] (struct req).filter_list renamed to vdp_filter_list
  *	order of vcl/vmod and director COLD events reversed to directors first
@@ -731,7 +760,7 @@ enum vcl_func_call_e {
 enum vcl_func_fail_e {
 	VSUB_E_OK,
 	VSUB_E_RECURSE, // call would recurse
-	VSUB_E_METHOD	// can not be called from this method
+	VSUB_E_METHOD	// cannot be called from this method
 };
 
 typedef void vcl_func_f(VRT_CTX, enum vcl_func_call_e, enum vcl_func_fail_e *);
@@ -754,10 +783,14 @@ struct vmod_data {
 	const char			*proto;
 	const char			*json;
 	const char			*abi;
+	const char			*vcs;
+	const char			*version;
 };
 
 /***********************************************************************
- * VCL events sent to VMODs
+ * Events sent to VMODs and directors:
+ * - VCL: VCL temperature events
+ * - VDI: director events
  */
 
 enum vcl_event_e {
@@ -765,6 +798,8 @@ enum vcl_event_e {
 	VCL_EVENT_WARM,
 	VCL_EVENT_COLD,
 	VCL_EVENT_DISCARD,
+
+	VDI_EVENT_SICK,
 };
 
 typedef int vmod_event_f(VRT_CTX, struct vmod_priv *, enum vcl_event_e);
@@ -856,8 +891,10 @@ struct vrt_endpoint {
 	vtim_dur			connect_timeout;	\
 	vtim_dur			first_byte_timeout;	\
 	vtim_dur			between_bytes_timeout;	\
+	vtim_dur			backend_wait_timeout;	\
 	unsigned			max_connections;	\
-	unsigned			proxy_header;
+	unsigned			proxy_header;		\
+	unsigned			backend_wait_limit;
 
 #define VRT_BACKEND_INIT(be)					\
 	do {							\
@@ -865,6 +902,7 @@ struct vrt_endpoint {
 		(be)->connect_timeout = -1.0;			\
 		(be)->first_byte_timeout = -1.0;		\
 		(be)->between_bytes_timeout = -1.0;		\
+		(be)->backend_wait_timeout = -1.0;		\
 	} while(0)
 
 #define VRT_BACKEND_HANDLE()			\
@@ -875,8 +913,10 @@ struct vrt_endpoint {
 		DN(connect_timeout);		\
 		DN(first_byte_timeout);		\
 		DN(between_bytes_timeout);	\
+		DN(backend_wait_timeout);	\
 		DN(max_connections);		\
 		DN(proxy_header);		\
+		DN(backend_wait_limit);		\
 	} while(0)
 
 struct vrt_backend {
